//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2012
//#
//#
//# File Name:  EPT_570_AP_U2_Top.v
//# Author:     R. Jolly
//# Date:       March 20, 2013
//# Revision:   A
//#
//# Development: USB Test Tool Interface board 
//# Application: Altera MAX II CPLD
//# Description: This file contains verilog code which will allow access
//#              to an eight bit bus in the FPGA. The FPGA receives its commands via
//#				 USB.
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			3/20/13 	A			Created			
//#                     			
//#
//#									
//#
//#######################################################################
`ifdef SIM
   `include "../src/define.v"
   `include "../Testbench/tb_define.v"
`endif

`timescale 1ns/1ps



//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_570_AP_U2_Top (

	
	input  wire [1:0]          aa,
	input  wire [1:0]          bc_in,
	output wire [2:0]          bc_out,
	inout  wire [7:0]		   bd_inout,
	
	input wire   [1:0]          LB_SER,       //XIOH -- J10
	input wire   [5:0]          LB_AD,        //AD   -- J9
	input wire   [7:0]          LB_IOH,       //XIOH -- J10
	output wire  [7:0]          LB_IOL,       //XIOL -- J8
	
	
	//Transceiver Control Signals
	output wire                TR_DIR_1,
	output wire                TR_OE_1,
	
	output wire                TR_DIR_2,
	output wire                TR_OE_2,

	output wire                TR_DIR_3,
	output wire                TR_OE_3,

	//input wire                SW_USER_1,
	//input wire                SW_USER_2,

	output wire [3:0]          LED
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------

    parameter                   GLOBAL_RESET_COUNT 			= 3'h7;
   
    // Block Transfer Options
	//parameter                  BLOCK_COUNT_16       		= 8'h10;

	//Block Transfer to Host State Machine
   parameter                   IDLE 						= 0,
                               STORE_FIFO_COUNT             = 1,
                               START_SEND_BLOCK				= 2,
							   FIFO_HI 						= 3,
							   FIFO_LO 						= 4,
							   INCR_COUNT 					= 5,
							   BLOCK_SEND_COMPLETE			= 6;


   // UART Parameter Declarations                        
   parameter                   UART_IDLE                    = 0,
                               RECEIVE_DATA				    = 1,
                               STORE_BYTE		    	    = 2,
                               SEND_TO_HOST			        = 3,
                               RECEIVE_COMPLETE			    = 4,
                               LOAD_SHIFT_REG				= 5,
                               WRITE_DATA_BYTE			    = 6,
                               WRITE_DATA_COMPLETE		    = 7;
		 
//	UART Clock Unit parameter. Use setting for 115,200
		parameter COUNT_TO_DECIMAL_2     = 8'h23; // 66 MHz

		// Fifo Data in Counter
	//parameter                  FIFO_DATA_COUNT = 8'h8;
	parameter                  FIFO_DATA_COUNT = 8'h01;

    // Test Registers
	parameter                  END_OF_BUFFER_COUNT = 4'h0f;
	
	//Store Fifo Counter
	parameter                  STATE_FIFO_COUNTER          = 4'h5;
	
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;
	
   	reg [7:0]                  state, next;

	//UART Inputs/Outputs
    wire                       UART_OUT; //To external 
    wire                       UART_IN; //From external

	// UART Storage Registers
	reg [7:0]                  rcv_storage_reg;
	//reg [2:0]                  store_reg_state; 
	
	//Clock Registers
	wire                       clk16x;
	
	
	// UART Metastable Registers
	reg                        uart_in_reg;
	reg                        usb_rxf_reg;
	
	//UART write_tx_byte_reg for signalling the UART transmitter
	//to start transmitting
	reg                        write_tx_byte_reg;

	//UART Interface signals
	wire [7:0]                  rx_data;
    wire                       rx_byte_present;
    wire                       UART_SERIAL_RX;
    wire                       en_16_x_baud;
    reg                        rx_byte_read;
	reg [7:0]                  tx_data_reg;
    wire                       tx_empty;
    reg                        write_tx_byte;
    //wire                       UART_SERIAL_TX;
	
	//Block Transfer Finite State Machine control registers
	reg 	[6:0]              state_out, next_out;

	//Trigger Signals
	reg [7:0]                  trigger_out;
	wire [7:0]                 trigger_in_byte;	
	
	//Control Registers
	wire                       start_stop_cntrl;
	
	//Main Reset register
	wire                       reset_cntrl;
	
	//Data in Pause control register
	wire                       data_in_pause;
	
	//Transfer registers
	reg                        transfer_out_reg;
	wire                       transfer_in_received;
	wire                       transfer_busy;
	wire [7:0]                 transfer_in_byte;
	//reg  [7:0]                 transfer_out_byte;
	//reg  [3:0]                 transfer_to_host_counter;

    //Transfer Control registers
	wire   [7:0]               control_register;

	
	//Block Transfer Out registers
	//reg [2:0]                  state_block_loopback;
	reg                        block_out_send;
	//wire                       block_out_received;
	reg [7:0]                  block_out_counter;
	//reg                        start_block_transfer;
	reg                        block_out_flag;
	wire                       block_byte_ready;
	wire [7:0]                 block_out_byte;
	//reg [1:0]                  block_transfer_out_state;
	
	//Block Transfer In registers
	wire [7:0]                 block_in_data;
	wire                       block_in_rcv;
	reg                        block_in_loopback;
	wire  [7:0]                ept_length;
	wire                       block_busy;

	//Block Transfer In FIFO registers
	reg                        fifo_rden;
	reg                        fifo_rden_reg;
	reg                        fifo_wren;
	reg                        fifo_wren_reg;
	reg                        fifo_wren_reg_2;
	wire                       fifo_wr_ack;
	wire [7:0]                 fifo_dout;
	wire                       fifo_empty;
	wire                       fifo_aempty;
	wire                       fifo_full;
	wire                       fifo_afull;
	reg                        fifo_flush;
	
	//Fifo Hold off register
	reg                        hold_off_flag;


	//Reset signals
	wire                       reset;
	reg [2:0]                 reset_counter;
	reg                        reset_signal_reg;
	
	//Fifo In Signals
	reg [7:0]                  fifo_in_counter;
	wire [7:0]                 fifo_data_in;
	//reg [7:0]                  fifo_data_in_counter;
	//reg                        fifo_in_counter_reg;
	reg [7:0]                  fifo_out_count;
	reg [7:0]                  store_fifo_counter;
	wire [7:0]                 block_fifo_count;
	
	//Test Registers
	reg                        end_of_buffer_reg;
	reg [3:0]                 end_of_buffer_counter;
	

`ifdef SIM
    reg [8*26:1] state_name;
    reg [8*26:1] state_name_out;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
  
//UART Clock control register  
`ifdef FAST_UART
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_2 - 1) );
`else
	reg	[7:0]			counter_reg;
//	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_3 - 1) );
	assign 	 clk16x = (counter_reg == (COUNT_TO_DECIMAL_2 - 1) );
`endif

   //UART Inputs/Outputs
   assign            LB_IOL[0] = UART_OUT;
   assign            LB_IOL[1] = 1'bz;
   assign            LB_IOL[2] = hold_off_flag | data_in_pause;
   assign            LB_IOL[3] = 1'bz;
 
    //Control Register Map
   assign            start_stop_cntrl = control_register[0];
   assign            reset_cntrl = control_register[1];
   assign            data_in_pause = control_register[2];


   assign            TR_DIR_1  = 1'b1; //1 = A to B; 0 = B to A
   //assign            TR_OE_1  = 1'b0;
   assign            TR_DIR_2  = 1'b1; //1 = A to B; 0 = B to A
   //assign            TR_OE_2  = 1'b0;
   assign            TR_DIR_3  = 1'b0; //1 = A to B; 0 = B to A

   assign            TR_OE_1  = start_stop_cntrl ? 1'b0 : 1'b1;
   assign            TR_OE_2  = start_stop_cntrl ? 1'b0 : 1'b1;
   assign            TR_OE_3  = (LB_AD[1] | state[LOAD_SHIFT_REG] | state[WRITE_DATA_BYTE] | state[WRITE_DATA_COMPLETE]) ? 1'b0: 1'b1;  
   //assign            TR_OE_3  = 1'b0;

   //Clock and Reset
   assign            CLK_66 = aa[1];
   assign            RST = reset;
   assign            reset = reset_signal_reg;
    
   //Block In Transfer
   assign            block_out_byte = fifo_dout;
   //assign            block_fifo_count = FIFO_DATA_COUNT | fifo_out_count;
       
  //Data input from Arduino to FIFO 
   assign            fifo_data_in = {LB_IOH[4],LB_SER[1],LB_IOH[3:0],LB_AD[4:3]};
   //assign            fifo_data_in = fifo_data_in_counter;
		 
   
   //LEDs 
   //assign            LED[3] = start_stop_cntrl ? 1'b0 : 1'bz;
   //assign            LED[2] = fifo_wren ? 1'b0 : 1'bz;
   //assign            LED[1] = fifo_rden ? 1'b0 : 1'bz;
   //assign            LED[0] = fifo_full ? 1'b0 : 1'bz;
   assign            LED[3] = start_stop_cntrl ? 1'b0 : 1'bz;
   assign            LED[2] = fifo_wren ? 1'b0 : 1'bz;
   assign            LED[1] = LB_AD[1] ? 1'b0 : 1'bz;
   assign            LED[0] = fifo_full ? 1'b0 : 1'bz;
   
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66 or negedge aa[0] or posedge reset_cntrl)
	begin
	  if(!aa[0])
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else if(reset_cntrl)
	  begin
	        reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // UART Transfer
   // Start the Transmit by asserting the write_tx_byte Signal 
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
		if (!RST)
		begin
			write_tx_byte <= 1'b1;
			write_tx_byte_reg <= 1'b0;
		end
		else 
		begin
			if(state[WRITE_DATA_BYTE] & !write_tx_byte_reg )
			begin
				write_tx_byte <= 1'b0;
				write_tx_byte_reg <= 1'b1;
			end
			else if( state[WRITE_DATA_COMPLETE] )
			begin
				write_tx_byte <= 1'b1;
				write_tx_byte_reg <= 1'b0;
		    end
		end	
	end // always @ (posedge CLK_66 or negedge RESET)   
   
   //-----------------------------------------------
   // UART Transfer
   // rx_byte_read register reads the available byte
   // in the UART receive module. 
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	begin
	     rx_byte_read <= 1'b1;
		 rcv_storage_reg <= 8'he5;
	end
	else
	  begin
	     if (state[STORE_BYTE])
		 begin
	         rx_byte_read <= 1'b0;
			 rcv_storage_reg <= rx_data;
		 end
		 else
		 begin
	         rx_byte_read <= 1'b1;
			 rcv_storage_reg <= rcv_storage_reg;
		 end
	  end
     end // always @ (posedge CLK_66 or negedge RST)

   //-----------------------------------------------
   // UART Transfer
   // Trigger In Byte bit 0 latches the value on the
   // Transfer-2 transfer_in_byte into the UART_TOP.
   // USB_RXF_REG asserts and the UART state machine 
   // goes to state[LOAD_SHIFT_REG] 
   // and starts the transmit process.
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	     usb_rxf_reg <= 1'b0;
	else
	  begin
	     if (trigger_in_byte[0])
		 begin
		     tx_data_reg <= transfer_in_byte;
	         usb_rxf_reg <= 1'b1;
	     end
		 else if(state[WRITE_DATA_BYTE])
	         usb_rxf_reg <= 1'b0;
	  end
     end // always @ (posedge CLK_66 or negedge RST)

   //-----------------------------------------------
   // UART Transfer
   // Register the UART_IN input with uart_in_reg. 
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	 uart_in_reg <= 1'b0;
	else
	  begin
	     if ( !UART_IN )
	       uart_in_reg <= 1'b0;//Hold the UART Receive in the off position
	     else 
	       uart_in_reg <= 1'b0;
	  end
     end // always @ (posedge CLK_66 or negedge RST)

	  
   //--------------------------------------------------
   // UART Transfer
   // UART State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 8'h00;
	     state[UART_IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

   always @ ( state or usb_rxf_reg or tx_empty or write_tx_byte
		 or uart_in_reg /*or store_reg_state*/ or rx_byte_present
		 or UART_OUT)
     begin
	next = 8'h00;

	if (state[UART_IDLE])
	begin
		if(usb_rxf_reg)
			next[LOAD_SHIFT_REG] = 1'b1;
		else
			next[UART_IDLE] = 1'b1;
	end

	if (state[LOAD_SHIFT_REG])
	       next[WRITE_DATA_BYTE] = 1'b1;

	if (state[WRITE_DATA_BYTE])
	  begin
	     if (!tx_empty)
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	     else
	       next[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_COMPLETE])
	  begin
	     if ( tx_empty )
	       next[UART_IDLE] = 1'b1;
	     else
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	  end

	
`ifdef SIM
	if (state == (1 << UART_IDLE))
	  state_name = "UART_IDLE";
	else if (state == (1 << RECEIVE_DATA))
	  state_name = "RECEIVE_DATA";
	else if (state == (1 << STORE_BYTE))
	  state_name = "STORE_BYTE";
	else if (state == (1 << SEND_TO_HOST))
	  state_name = "SEND_TO_HOST";
	else if (state == (1 << RECEIVE_COMPLETE))
	  state_name = "RECEIVE_COMPLETE";
	else if (state == (1 << LOAD_SHIFT_REG))
	  state_name = "LOAD_SHIFT_REG";
	else if (state == (1 << WRITE_DATA_BYTE))
	  state_name = "WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_COMPLETE))
	  state_name = "WRITE_DATA_COMPLETE";
`endif
     end // always @ (...
   

   //-----------------------------------------------
   // BLock Transfer
   // End Of Buffer Signal informs user code
   // that the Oscilloscope has sent the entire
   // buffer to the CPLD.
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			end_of_buffer_reg <= 1'b0;
			end_of_buffer_counter <= 0;
	  end
	  else
	  begin
		  if(start_stop_cntrl)
		  begin
		     if(LB_AD[2] & !end_of_buffer_reg)
			 begin
			     end_of_buffer_reg <= 1'b1;
			 end
			 else if(end_of_buffer_reg)
			 begin
			     if(end_of_buffer_counter < END_OF_BUFFER_COUNT )
				     end_of_buffer_counter <= end_of_buffer_counter + 1'b1;
				 else
				 begin
				     end_of_buffer_reg <= 1'b0;
				     end_of_buffer_counter <= 0;
			     end
			 end
		  end
	  end
	end

   //-----------------------------------------------
   // BLock Transfer
   // Trigger To Host
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
     if(!RST)
	 begin
	     trigger_out <= 8'h00;
	 end
	else 
	begin
	   if(end_of_buffer_reg)
	        trigger_out <= 8'h04;
	    else 
	        trigger_out <= 8'h00;
	end
  end

 
   //-----------------------------------------------
   // BLock Transfer
   // Block In write fifo enable is driven by block_byte_ready
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
        // fifo_data_in_counter <= 0;


         fifo_wren <= 1'b0;
		 fifo_wren_reg <= 1'b0;
		 fifo_wren_reg_2 <= 1'b0;
		 //fifo_in_counter <= 0;
	end
	else 
	begin
	  if(start_stop_cntrl)
	  begin
	     //if(state_out[START_SEND_BLOCK])
		 //begin
		//	     fifo_in_counter <= 0;
		 //end
	     if(LB_AD[5] & !fifo_wren_reg )//& !fifo_wren_reg_2)
		 begin
		 
		     fifo_wren <= 1'b1;
		     fifo_wren_reg <= 1'b1;
		//	 fifo_in_counter <= fifo_in_counter + 1'd1;
		 end
	     else if(LB_AD[5] & fifo_wren_reg )//& !fifo_wren_reg_2)
		 //else if(fifo_wr_ack)
		 begin
		     fifo_wren <= 1'b0;
		     //fifo_wren_reg <= 1'b0;
		     //fifo_wren_reg_2 <= 1'b1;
		 end
	     else if(!LB_AD[5] & fifo_wren_reg)//fifo_wren_reg_2)
		 begin
		     //if(fifo_data_in_counter < 8'hff)
		     //   fifo_data_in_counter <= fifo_data_in_counter + 1'd1;
		     //else 
			 //   fifo_data_in_counter <= 0;
		 
		     fifo_wren <= 1'b0;
		     fifo_wren_reg <= 1'b0;
		 //    fifo_wren_reg_2 <= 1'b0;
		 end
		 //else
		 //begin
		     //if(fifo_in_counter >= FIFO_DATA_COUNT)
		//	 if(state_out[START_SEND_BLOCK])
		//	     fifo_in_counter <= 0;
		 //end
	  end
	  else
	  	begin
		 fifo_wren <= 1'b0;
		 fifo_wren_reg <= 1'b0;
		 //fifo_in_counter <= 0;
         //fifo_wren_reg_2 <= 1'b0;
	    end
	end 
  end
 
   //-----------------------------------------------
   // BLock Transfer
   // Flush the Fifo After transferring all data to
   // the PC
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
     if(!RST)
	 begin
          fifo_flush <= 1'b0;
	 end
	else 
	begin
	   if(state_out[BLOCK_SEND_COMPLETE] | end_of_buffer_reg)
			fifo_flush <= 1'b1;
	    else 
			fifo_flush <= 1'b0;
	end
  end

   //-----------------------------------------------
   // BLock Transfer
   // Assert the block_out_send when in state_out[START_SEND_BLOCK] 
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     block_out_send <= 1'b0;
	end
	else
	begin
	  if(state_out[START_SEND_BLOCK])
	  begin
	    block_out_send <= 1'b1;
      end
      else if(state_out[INCR_COUNT])
      begin
	     block_out_send <= 1'b0;
      end
    end
  end
  
   //-----------------------------------------------
   // BLock Transfer
   // Increment the block_out_count in  state_out[INCR_COUNT]
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     block_out_counter <= 0;
		 block_out_flag <= 1'b0;
		 fifo_out_count <= 0;
	end
	else
	begin
	  if(state_out[STORE_FIFO_COUNT])
	  begin
         //fifo_out_count <= fifo_in_counter;
         fifo_out_count <= 8'h08;
      end
	  else if(state_out[INCR_COUNT] & !block_out_flag)
	  begin
         block_out_counter <= block_out_counter + 1'd1;
		 if(block_out_counter < (fifo_out_count - 1'd1))
	         block_out_flag <= 1'b1;
      end
      else if(state_out[FIFO_HI])
      begin
		 block_out_flag <= 1'b0;
      end
      else if(state_out[BLOCK_SEND_COMPLETE])
      begin
	     block_out_counter <= 1'b0;
		 block_out_flag <= 1'b0;
		 fifo_out_count <= 0;
      end
    end
  end
  
/*  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     block_out_counter <= 1'b0;
		 block_out_flag <= 1'b0;
	end
	else
	begin
	  if(state_out[INCR_COUNT] & !block_out_flag)
	  begin
         block_out_counter <= block_out_counter + 1'd1;
		 if(block_out_counter < (FIFO_DATA_COUNT - 1'd1))
	         block_out_flag <= 1'b1;
      end
      else if(state_out[FIFO_HI])
      begin
		 block_out_flag <= 1'b0;
      end
      else if(state_out[BLOCK_SEND_COMPLETE])
      begin
		 block_out_flag <= 1'b0;
	     block_out_counter <= 1'b0;
      end
    end
  end
*/  

   //-----------------------------------------------
   // BLock Transfer
   // hold_off_flag register used to prevent the Arduino
   // from transmitting data to the UnoProLogic   
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     hold_off_flag <= 1'b0;
	end
	else
	begin
	  //if(fifo_afull)
	  if(fifo_full)
	  begin
	     hold_off_flag <= 1'b1;
      end
      else if(fifo_aempty & hold_off_flag)
      begin
	     hold_off_flag <= 1'b0;
      end
    end
  end
 
   //-----------------------------------------------
   // BLock Transfer
   // store_fifo_counter to delay the  state_out[STORE_FIFO_COUNT]
   // until fifo_in_count is stored   
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     store_fifo_counter <= 0;
	end
	else
	begin
	   if(state_out[STORE_FIFO_COUNT])
	   begin
 	     if(store_fifo_counter < STATE_FIFO_COUNTER)
		 begin
		     store_fifo_counter <= store_fifo_counter + 1'd1;
 		 end
         else
		 begin
		    store_fifo_counter <= 0;
		 end
	   end
	end
  end
 
   //--------------------------------------------------
   // BLock Transfer
  // Block-FIFO Outgoing Data State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state_out <= 7'h00;
	     state_out[IDLE] <= 1'b1;
	  end
	else
	  state_out <= next_out;
     end

   always @ ( fifo_in_counter or block_busy or block_byte_ready
	     or fifo_rden or block_out_flag or state_out or block_out_counter 
		 or fifo_out_count or fifo_full or fifo_aempty
		 )
   begin
	next_out = 7'h00;

	if (state_out[IDLE])
	begin
        //if((fifo_in_counter >= FIFO_DATA_COUNT) & !block_busy)
        //if(fifo_afull & !block_busy)
        if(fifo_full & !block_busy)
			next_out[STORE_FIFO_COUNT] = 1'b1;
		else
			next_out[IDLE] = 1'b1;
	end

    if (state_out[STORE_FIFO_COUNT])
	begin
	     //if(store_fifo_counter < STATE_FIFO_COUNTER)
         //    next_out[STORE_FIFO_COUNT] = 1'b1;
        //else
            next_out[START_SEND_BLOCK] = 1'b1;
    end

    if (state_out[START_SEND_BLOCK])
    begin
        if(block_busy)
            next_out[FIFO_HI] = 1'b1;
	    else
            next_out[START_SEND_BLOCK] = 1'b1;
    end

	if (state_out[FIFO_HI])
	begin
       if(block_byte_ready)
       begin
           fifo_rden <= 1'b1;
           next_out[FIFO_LO] = 1'b1;
       end
       else
       begin
           fifo_rden <= 1'b0;
           next_out[FIFO_HI] = 1'b1;
       end
    end

	if (state_out[FIFO_LO])
    begin
       if(block_byte_ready)
       begin
          fifo_rden <= 1'b0;
          next_out[FIFO_LO] = 1'b1;
       end
       else if(!block_byte_ready )
       begin
          fifo_rden <= 1'b0;
          next_out[INCR_COUNT] = 1'b1;
       end
    end

	if (state_out[INCR_COUNT])
	  begin
         if(block_out_flag)
	       next_out[FIFO_HI] = 1'b1;
         else if(block_out_counter >= fifo_out_count )
	       next_out[BLOCK_SEND_COMPLETE] = 1'b1;
         else 
	       next_out[INCR_COUNT] = 1'b1;
	  end

	if (state_out[BLOCK_SEND_COMPLETE])
	  begin
         if(block_busy)
	       next_out[BLOCK_SEND_COMPLETE] = 1'b1;
         else 
	       next_out[IDLE] = 1'b1;
	  end

	
`ifdef SIM
	if (state_out == (1 << IDLE))
	  state_name_out = "IDLE";
	else if (state_out == (1 <<  STORE_FIFO_COUNT))
	  state_name_out = "STORE_FIFO_COUNT";
	else if (state_out == (1 << START_SEND_BLOCK))
	  state_name_out = "START_SEND_BLOCK";
	else if (state_out == (1 << FIFO_HI))
	  state_name_out = "FIFO_HI";
	else if (state_out == (1 << FIFO_LO))
	  state_name_out = "FIFO_LO";
	else if (state_out == (1 << INCR_COUNT))
	  state_name_out = "INCR_COUNT";
	else if (state_out == (1 << BLOCK_SEND_COMPLETE))
	  state_name_out = "BLOCK_SEND_COMPLETE";
`endif
     end // always @ (...
		
  
//==============================================================================
   // UART Transfer
//	Instantiate the 16x Clock Counter
//==============================================================================
   always @(posedge CLK_66 or negedge RST)
   begin
	if (!RST)
 	  counter_reg <= 0;
 	else
 	  begin
 	     if ( counter_reg == COUNT_TO_DECIMAL_2 )
 	       counter_reg <= 0;
 	     else
 	       counter_reg <= counter_reg + 1;
 	  end
   end // always @ (posedge CLK_66 or negedge RST)

   //----------------------------------------------
   // UART Transfer
   // Instantiate the UART_top Leaf FSM Modules
   //----------------------------------------------
   uart_top uart_inst
     (
      .CLK                    (CLK_66),
      .RST_N                  (RST),
      .DOUT                   (rx_data),
      .DATA_READY             (rx_byte_present),
      .RXD                    (UART_IN),
      .CLK16X                 (clk16x),
      .RDN                    (rx_byte_read),
      .DIN                    (tx_data_reg),
      .TX_BUFFER_EMPTY        (tx_empty),
      .WRN                    (write_tx_byte),
      .SDO                    (UART_OUT)
      );

   //-----------------------------------------------
   // State Machine: Control Register from Transfer In 
   //-----------------------------------------------
  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_66),
   .RST                        (RST),
   .TRANSFER_IN_RECEIVED       (transfer_in_received),
   .TRANSFER_IN_BYTE           (transfer_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
  
   //-----------------------------------------------
   // Block In Sync FIFO stores bytes from the Host
   //-----------------------------------------------
  sync_fifo #(
      .ADDR_WIDTH               (3),
	  .DATA_WIDTH               (8),
	  .DEPTH                    (8),
	  .AEMPTY                   (1),
	  .AFULL                    (6)
	  ) BLOCK_IN_FIFO
     (
	  .clk                     (CLK_66),
	  .reset_n                 (RST),
	  .flush                   (fifo_flush),
	  .write_data              (fifo_data_in),	  
	  .read_req                (fifo_rden),
	  .wdata_valid             (fifo_wren),
	  .read_data               (fifo_dout),
	  .rdata_valid             (),
	  .fifo_empty              (fifo_empty),
	  .fifo_aempty             (fifo_aempty),
	  .fifo_full               (fifo_full),
	  .fifo_afull              (fifo_afull),
	  .write_ack               (fifo_wr_ack)
      );


   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_transfer_library	   ACTIVE_TRANSFER_LIBRARY_INST
	(	
	.aa                        (aa),
	.bc_in                     (bc_in),
	.bc_out                    (bc_out),
	.bd_inout                  (bd_inout),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------
wire [22*3-1:0]  uc_out_m;
eptWireOR # (.N(3)) wireOR (UC_OUT, uc_out_m);
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

	active_transfer            ACTIVE_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (transfer_out_reg),
	 .transfer_received        (transfer_in_received),
	
	 .transfer_busy            (transfer_busy),

	 .uc_addr                  (3'h2),

	 .transfer_to_host         (/*transfer_out_byte*/),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (block_out_send),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (ept_length),
	
	 .uc_addr                  (3'h4),
	 
	 //.uc_length                (FIFO_DATA_COUNT),
	 .uc_length                (fifo_out_count),
	 //.uc_length                (block_fifo_count),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       (block_in_data)
	
	);

	
endmodule
